//
//  BindCommand.swift
//  EscapePackageDescription
//
//  Created by SalmaAli on 7/15/18.
//

import Foundation
import SwiftCLI
import SwiftShell
import FileUtils

class BindCommand: Command {
    let name = "bind"
    let shortDescription = "Binds native binaries to xamarin binding libraries."
    
    var ios = Flag("-i", "--ios", description: "Binds native framework to iOS package")
    var android = Flag("-a", "--android", description: "Binds a native aar to a xamarin Binding Android Library.")
    
    var optionGroups: [OptionGroup] {
        let platform: OptionGroup = .exactlyOne(ios, android)
        return [platform]
    }
    let projectDirectory = main.currentdirectory
    
    init() {
        
    }
    
    init( ios: Flag, android: Flag) {
        self.android = android
        self.ios = ios
    }
    
    func execute() throws {
        if (android.value) {
            bind(platform: Constants.platforms.android)
        } else {
            bindIOS()
        }
    }
    
    func bindIOS() {
        let repoPath = Constants.cacheDirectoryPathFor(platform: Constants.platforms.iOS)
        if Path.exists("\(repoPath).git") {
            let iOSModule = Constants.xamariniOSModule
            let copySuccess = Helpers.copy(fileAt: "\(repoPath)/Instabug.framework", toPath: "\(projectDirectory)\(iOSModule)/\(iOSModule)/IBGFrameworks/Instabug.framework")
            if (!copySuccess){
                exit(1)
            }
            Logger.printSuccess("Copied iOS framework successfuly.")
        } else {
            Logger.printError("\( Constants.xamariniOSModule) project does not exist. Try fetching the project first.")
        }
        
    }
    
    func bind(platform: String) {
       
        let repoPath = Constants.cacheDirectoryPathFor(platform: platform)
        
        if Path.exists("\(repoPath).git") {
            let copyBinariesSuccess = copyBinaries(repoPath: repoPath)
            if (copyBinariesSuccess) {
                Logger.printSuccess("Copied all Android modules successfuly.")
            }
        } else {
            Logger.printError("\(platform) project does not exist. Try fetching the project first.")
        }
    }
    
    private func copyBinaries(repoPath: String) -> Bool {
        
        let modules = Constants.xamarinAndroidModules
        for (module, xamarinModule) in modules {
            let aarPath = "\(repoPath)\(module)/build/outputs/aar"
            if (Path.exists(aarPath)) {
                let aarName = "\(module)-release.aar"
                let copySuccess = Helpers.copy(fileAt: "\(aarPath)/\(aarName)", toPath: "\(projectDirectory)\(xamarinModule)/\(xamarinModule)/Jars/\(aarName)")
                if (!copySuccess){
                    exit(1)
                }
            } else {
                Logger.printError("Library aar path for the module \(module) does not exist in path \(aarPath). Try building the project first.")
                return false
            }
        }
        return true
        
    }
    
}
