//
//  BuildCommand.swift
//  EscapePackageDescription
//
//  Created by SalmaAli on 7/15/18.
//

import Foundation
import SwiftCLI
import SwiftShell
import FileUtils

class BuildCommand: Command {
    let name = "build"
    let shortDescription = "Builds the native libraries."
    
    var android = Flag("-a", "--android", description: "Builds Android aar.")
    
    init() {
        
    }
    
    init(android: Flag) {
        self.android = android
    }
    
    func execute() throws {
        if (android.value) {
            build(platform: Constants.platforms.android)
        }
    }
    
    func build(platform: String) {
        let repoPath = Constants.cacheDirectoryPathFor(platform: platform)
    
        if Path.exists("\(repoPath).git") {
            let cacheDirectoryPath = Constants.cacheDirectoryPath
            let repoPath = cacheDirectoryPath.appending(platform)
            
            Logger.print("Building repo at \(repoPath)")
            
            main.currentdirectory = repoPath
            Logger.printMessage("Current Directory: "+main.currentdirectory)

            do {
                try "".runAndPrint("./gradlew", "clean")
                let modules = Constants.xamarinAndroidModules
                for(module, _) in modules {
                    do {
                        try "".runAndPrint(bash: "./gradlew :\(module):assembleRelease")
                    } catch {
                        Logger.printFatalError("Failed to build \(module) module in Android.")
                    }
                }
            } catch {
                Logger.printFatalError("An error occurred while cleaning the Android project.")
            }
        } else {
            Logger.printError("\(platform) project does not exist. Try fetching the project first.")
        }
    }
}
