//
//  ChangeLogCommand.swift
//  Escape
//
//  Created by Salma Ali on 9/27/18.
//

import Foundation
import SwiftCLI
import SwiftShell
import FileUtils
import Networking

class ChangeLogCommand : Command {
    
    let name = "changelog"
    let shortDescription = "Calls change log api to post changes to dashboard."
    var version = Key<String>("-v", "--version", description: "The version released.")
    var releaseDate = Key<String>("-d", "--release-date", description: "The release date.")
    var releaseNotes = Key<String>("-r", "--release-notes", description: "The release notes.")
    private let networking = Networking(baseUrl: URL(string: "https://api.instabug.com/")!)

    var framework: Int
    
    
    init (framework: Int) {
        self.framework = framework
    }
    
    init (framework: Int, releaseDate: Key<String>, releaseNotes: Key<String>, version: Key<String>) {
        self.framework = framework
        self.version = version
        self.releaseDate = releaseDate
        self.releaseNotes = releaseNotes
    }
    
    func execute() throws {
        switch framework {
        case Constants.reactNative:
            postChangleLog(withFramework: Constants.reactNative, withRemoteUrl: Constants.rnRemoteUrl, withAppRemoteUrl: Constants.rnSampleAppUrl)
            break
        case Constants.xamarin:
            postChangleLog(withFramework: Constants.xamarin, withRemoteUrl: "", withAppRemoteUrl: "")
        default:
            postChangleLog(withFramework: 0, withRemoteUrl: "", withAppRemoteUrl: "")
            
        }
    }
    
    private func postChangleLog(withFramework framework: Int, withRemoteUrl remoteUrl: String, withAppRemoteUrl appUrl: String) {
        
        let parameters = [
            "token": "st@yfo0lish",
            "change_log": [
                "version": version.value!,
                "release_date": releaseDate.value!,
                "target_os": 4,
                "sdk_changes": releaseNotes.value!
            ],
            "sdk_remote_url": remoteUrl,
            "sample_app_remote_url": appUrl
            ] as [String : Any]
        
        let networkResponse = networking.postSynchronously(path: "api/internal/change_logs", parameters: parameters)
        guard networkResponse.response?.statusCode == 200 else {
            Logger.printError("Something went wrong!")
            return
        }
        Logger.print("Changelogs post to dashboard successfully.")
        
    }

}
