//
//  CustomBuildCommand.swift
//  Escape
//
//  Created by Salma Ali on 1/8/19.
//

import Foundation
import SwiftCLI

class CustomBuildCommand : Command {
    let name: String = "custom-build"
    let shortDescription: String = "Create custom build from a native SDK custom build."
    let snapshot = Key<String>("-s", "--snapshot", description: "The snaphot of the custom Android SDK.")
    let binary = Key<String>("-b", "--binary", description: "The path to the iOS custom binary.")
//    let
    let issueLink = Key<String>("-i", "--issue", description: "The link of the support/github issue")
    
    func execute() throws {
        if (issueLink.value == nil) {
            throw CLI.Error(message: "Missing value issue")
        }
        if (snapshot.value != nil) {
            updateGradleVersion()
        }
        if (binary.value != nil) {
            updateiOSBinary()
        }
        try generateCustomBuild()
    }
    
    private func updateGradleVersion() {
        let gradleRegex = "com.instabug.library:instabug:[1-9]+.[0-9]+.[0-9]+"
        var gradleFile = Helpers.openFileContents(fromPath:  "\(Constants.rnGradlePath)")
        Logger.print("\(Constants.rnGradlePath)")
        guard let instabugGradleCommand = Helpers.matches(for: gradleRegex, in: gradleFile) else {
            Logger.printFatalError("No gradle commands were found for Instabug in the build.gradle!")
            return
        }
        gradleFile = gradleFile.replacingOccurrences(of: "\(instabugGradleCommand)", with: "com.instabug.library:instabug:\(snapshot.value!)")
        Helpers.write(fileContents: gradleFile, toPath: "\(Constants.rnGradlePath)")
    }
    
    private func updateiOSBinary() {
        let copySuccess = Helpers.copy(fileAt: binary.value!, toPath: "ios")
        if(!copySuccess) {
            exit(1)
        } else {
            Logger.printSuccess("Framework copied successfully")
        }
    }
    
    private func generateCustomBuild() throws {
        let issueNumber = issueLink.value?.components(separatedBy: "/").last
        let branchName = "customBuild/\(issueNumber!)"
        let commitMessage = "🚑 Custom build for issue \(issueLink.value!)"
        do {
            try "".runAndPrint("/usr/bin/git checkout master")
            try "".runAndPrint("/usr/bin/git", "branch", branchName)
            try "".runAndPrint("/usr/bin/git", "commit", "-m", commitMessage)
            try "".runAndPrint("/usr/bin/git", "push", "origin", branchName)
            Logger.printSuccess("Custom build created successfully!")
            Logger.print("To install run")
            Logger.print("npm install --save https://github.com/Instabug/Instabug-React-Native#\(branchName)")
            Logger.print("If you're running Android add the following to your project level build.gradle under allProjects -> repositories")
            Logger.print("maven { url \"https://oss.sonatype.org/content/repositories/snapshots\" }")
        } catch {
            throw CLI.Error(message: "An error occurred while creating custom build")
        }
    }
    
    
    
    
}
