import Foundation
import SwiftCLI
import SwiftShell
import FileUtils

class FetchCommand: Command {
    let name = "fetch"
    let shortDescription = "Fetches the latest version of an SDK."
    let projectDirectory = main.currentdirectory;
    
    var ios = Flag("-i", "--ios", description: "Fetch iOS public repo.")
    var android = Flag("-a", "--android", description: "Fetches Android repo.")
    var optionGroups: [OptionGroup] {
        let platform: OptionGroup = .exactlyOne(ios, android)
        return [platform]
    }
    var xamarin = Flag("-x", "--xamarin", description: "Checkout xamarin android branch.")
    
    init() {
        
    }
    
    init(ios: Flag, android: Flag) {
        self.ios = ios
        self.android = android
    }
    
    func execute() throws {
        if ios.value {
            fetch(platform: Constants.platforms.iOS, repositoryName: Constants.repositoryNames.iOS)
            //copyiOSFrameworks()
            
        } else {
           fetch(platform: Constants.platforms.android, repositoryName: Constants.platforms.android)
//
//            if xamarin.value {
                let repoPath = Constants.cacheDirectoryPathFor(platform: Constants.platforms.android)
                let branchName = Constants.androidBranchForXamarin
                checkoutBranch(repoPath: repoPath, branchName: branchName)
           // }
        }
    }
    
    func fetch(platform: String, repositoryName: String) {
        let repoPath = Constants.cacheDirectoryPathFor(platform: platform)
        let repoName = Constants.repositoryNames.repositoryNameFor(platform: platform)

        if Path.exists("\(repoPath).git") {
            pull(repoPath: repoPath)
        } else {
            let options = platform == Constants.platforms.iOS ? "--depth 1 --no-single-branch" : ""
            clone(repoName: repoName, cloneOptions: options)
        }
    }
    
    func pull(repoPath: String) {
        Logger.print("Pulling latest changes at \(repoPath)")
        main.currentdirectory = repoPath
        do {
            try "".runAndPrint(bash: "/usr/bin/git pull")
        } catch {
            Logger.printFatalError("An error occurred while pulling changes at \(repoPath)")
            exit(1)
        }
    }
    
    func checkoutBranch(repoPath: String, branchName: String) {
        Logger.print("Checking out to branch \(branchName)")
        
        main.currentdirectory = repoPath
        let command = main.run(bash: "git checkout \(branchName)")
        
        if command.error != nil {
            Logger.printError(command.stderror)
            Logger.printFatalError("An error occurred while checking out to \(branchName)")
            exit(1)
        } else {
            Logger.printSuccess("Completed checking out to branch \(branchName)")
        }

//        let mergeCommand = main.run(bash: "/usr/bin/git merge  master -m \"Merge master into branch\"")
//        if mergeCommand.error != nil {
//            Logger.printError(command.stderror)
//            Logger.printFatalError("Resolve merge conflicts.")
//            exit(1)
//        } else {
//            Logger.printSuccess("Merged master into branch successfully")
//            let pushCommand = main.run(bash: "/usr/bin/git push")
//            if pushCommand.error != nil {
//                Logger.printError(command.stderror)
//                Logger.printFatalError("An error occurred while pushing after merge")
//            } else {
//                Logger.printSuccess("Pushed merged changes successfully")
//            }
//        }
        
    }
    
    func clone(repoName: String, cloneOptions: String) {
        let cacheDirectoryPath = Constants.cacheDirectoryPath
        Logger.print("Cloning \(repoName) repo into \(cacheDirectoryPath)\(repoName)")
        
        main.currentdirectory = cacheDirectoryPath
        do {
            try "".runAndPrint(bash: "/usr/bin/git clone https://github.com/Instabug/\(repoName).git " + cloneOptions)
            Logger.printSuccess("Completed fetching \(repoName) into \(cacheDirectoryPath)")
        } catch {
            Logger.printFatalError("An error occurred while fetching \(repoName) into \(cacheDirectoryPath)")
            exit(1)
        }
    }
    
    private func copyiOSFrameworks() {
        let repoName = Constants.repositoryNames.repositoryNameFor(platform: Constants.platforms.iOS)
    
        var toPath = "\(projectDirectory)/\(ios)"
        if (xamarin.value) {
            toPath = "\(projectDirectory)/\(Constants.xamariniOSModule)/\(Constants.xamariniOSModule)/IBGFrameworks/\(Constants.iOSFramework)"
        }
        
        let copySuccess = Helpers.copy(fileAt: "\(Constants.cacheDirectoryPath)/\(repoName)/\(Constants.iOSFramework)", toPath: toPath)
        if(!copySuccess) {
            exit(1)
        } else {
            Logger.printSuccess("Framework copied successfully")
        }
    }
}
