//
//  BindCommand.swift
//  EscapePackageDescription
//
//  Created by SalmaAli on 7/15/18.
//

import Foundation
import SwiftCLI
import SwiftShell
import FileUtils

class PackCommand: Command {
    let name = "pack"
    let shortDescription = "Creates Nuget packages for the xamarin binding libraries."
    
    // Flags for the command
    var android = Flag("-a", "--android", description: "Creates Nuget packages for all the android modules.")
    var ios = Flag("-i", "--ios", description: "Creates Nuget packages for the iOS module.")
    var newVersion = Key<String>("-v", "--version", description: "Enters the new version for the nuget package.")
    
    var releaseNotes:String
    
    // Makes sure that only one flag either ios or android is there
    var optionGroups: [OptionGroup] {
        let platform: OptionGroup = .exactlyOne(ios, android)
        return [platform]
    }
    
    // Constantly have access to the project directory
    let projectDirectory = main.currentdirectory;
    
    init() {
        self.releaseNotes = ""
    }
    
    init(ios: Flag, android: Flag, newVersion: Key<String>, releaseNotes: String) {
        self.ios = ios
        self.android = android
        self.newVersion = newVersion
        self.releaseNotes = releaseNotes
    }
    
    func execute() throws {
        if android.value {
           editNuspecFile(module: "\(Constants.xamariniAndroidMainModule)/\(Constants.xamariniAndroidMainModule)/\(Constants.xamariniAndroidMainModule)")
        } else if ios.value {
            editNuspecFile(module: "\(Constants.xamariniOSModule)/\(Constants.xamariniOSModule)/\(Constants.xamariniOSModule)")
        }
    }

    private func editNuspecFile(module: String) {
        do {
            
            // Reading contents of nuspec file
            var nuspecFileContents = Helpers.nuspecFileContents(from: main.currentdirectory.appending("\(module).nuspec"))
            
            // Writable for .nuspec file for editing.
            let nuspecFileWritable = try open(forWriting: main.currentdirectory.appending("\(module).nuspec"), overwrite: true, encoding: main.encoding)
            
            // Acquiring the version and release notes from the previous version
            let version = Helpers.versionNumber(from: nuspecFileContents)
            let releaseNotesTag = Helpers.releaseNotes(from: nuspecFileContents)
            
            // Update the version and release notes
            nuspecFileContents = nuspecFileContents.replacingOccurrences(of: "\(version)", with: "<version>\(newVersion.value!)</version>")
            nuspecFileContents = nuspecFileContents.replacingOccurrences(of: "\(releaseNotesTag)", with: "<releaseNotes>\(releaseNotes)</releaseNotes>")
            
            // Write the new .nuspec file
            nuspecFileWritable.write(nuspecFileContents)
            nuspecFileWritable.close()
        } catch {
            Logger.printError("Error writing to file")
        }
    }
}
