//
//  ReleaseCommand.swift
//  Escape
//
//  Created by Salma Ali on 9/9/18.
//

import Foundation
import SwiftCLI
import SwiftShell

class XamarinReleaseCommand: Command {
    let name = "release"
    let shortDescription = "Fetches the latest Native SDKS, build, binds and packs then commits these changes to the repo and creates a pull request"
    let newVersion = Key<String>("-v", "--version", description: "Enters the new version for the nuget package.")
    
    func execute() throws {
        let falseFlag = Flag()
        let trueFlag = Flag()
        trueFlag.toggle()
        let xamarinProjectDirectory = main.currentdirectory
        
        // Fetching iOS and android native repos
        try FetchCommand(ios: falseFlag, android: trueFlag).execute()
        try FetchCommand(ios: trueFlag, android: falseFlag).execute()
        
        // Building native android project
        try BuildCommand(android: trueFlag).execute()
        
        main.currentdirectory = xamarinProjectDirectory
        try "".runAndPrint(bash: "/usr/bin/git checkout -b \(newVersion.value!)")
        
        // Binds aar libraries to corresponding android packages
        try BindCommand(ios: falseFlag, android: trueFlag).execute()
        // Binds native framework to iOS Package
        try BindCommand(ios: trueFlag, android: falseFlag).execute()
        
        let changelog = Helpers.getChangelog();
        var markdownChangelog = "";
        for change in changelog {
            markdownChangelog += change + "\n";
        }
        try PackCommand(ios: falseFlag, android: trueFlag, newVersion: newVersion, releaseNotes: markdownChangelog).execute()
        try PackCommand(ios: trueFlag, android: falseFlag, newVersion: newVersion, releaseNotes: markdownChangelog).execute()
        try "".runAndPrint(bash: "/usr/bin/git add .")
        try "".runAndPrint(bash: "/usr/bin/git commit -m \(newVersion.value!)")
        try "".runAndPrint(bash: "/usr/bin/git push origin \(newVersion.value!)")
        
        try "".runAndPrint("curl", "-f", "--data", "{\"title\": \"Bumping to \(newVersion.value!)\", \"body\": \"Initial pull request with the new native libraries\", \"head\": \"\(newVersion.value!)\", \"base\": \"master\"}", "https://api.github.com/repos/Instabug/instabug-xamarin/pulls?access_token=\(Constants.githubToken)");
    }
}

class RNReleaseCommand: Command {
    let name = "release"
    let shortDescription = "Releases new instabug library."
    let newVersion = Key<String>("-v", "--version", description: "Enters the new version for the npm package.")
    let releaseNotes = Key<String>("-r", "--releasenotes", description: "Enters the release notes for the npm package.")
    let releaseDate = Key<String>("-d", "--releasedate", description: "Enters the release date for the npm package.")
    
    
    func execute() throws {
        // fetch both repos here
        let trueFlag = Flag()
        trueFlag.toggle()
        try FetchCommand(ios: trueFlag, android: trueFlag).execute()
        try UpdateCommand().execute()
        try RNPublishCommand().execute()
        try ChangeLogCommand(framework: Constants.reactNative).execute()
    }
}
