//
//  UpdateCommand.swift
//  Escape
//
//  Created by Salma Ali on 9/26/18.
//

import Foundation
import SwiftCLI
import SwiftShell
import FileUtils

class UpdateCommand: Command {
    let name = "update"
    let shortDescription = "Updates native android gradle used to latest."
    let projectDirectory = main.currentdirectory;
    
    init() {
    }
    
    func execute() throws {
        updateGradleVersion()
        pushChangesToRemote()
    }
    
    private func pushChangesToRemote() {
        do {
            try "".runAndPrint("/usr/bin/git", "checkout", "master")
            try "".runAndPrint("/usr/bin/git", "add", "android/")
            try "".runAndPrint("/usr/bin/git", "add", "ios/")
            try "".runAndPrint("/usr/bin/git", "commit", "-m", "\"⬆️ Upgrade native Android and iOS SDKs\"")
            try "".runAndPrint("/usr/bin/git", "push", "origin", "master")
        } catch {
            
        }
    }
    
    private func updateGradleVersion() {
        let gradleRegex = "com.instabug.library:instabug:[1-9]+.[0-9]+.[0-9]+"
        var gradleFile = Helpers.openFileContents(fromPath:  "\(projectDirectory)/\(Constants.rnGradlePath)")
        Logger.print("\(projectDirectory)/\(Constants.rnGradlePath)")
        guard let instabugGradleCommand = Helpers.matches(for: gradleRegex, in: gradleFile) else {
            Logger.printFatalError("No gradle commands were found for Instabug in the build.gradle!")
            return
        }
        let version = getNewVersion()
        gradleFile = gradleFile.replacingOccurrences(of: "\(instabugGradleCommand)", with: "com.instabug.library:instabug:\(version)")
        Helpers.write(fileContents: gradleFile, toPath: "\(projectDirectory)/\(Constants.rnGradlePath)")
    }
    
    private func getNewVersion() -> String {
        let versionNameRegex = "VERSION_NAME=[1-9]+.[0-9]+.[0-9]+"
        let gradlePropertiesContents = Helpers.openFileContents(fromPath: "\(Constants.cacheDirectoryPathFor(platform: Constants.platforms.android))/gradle.properties")
        guard let versionName = Helpers.matches(for: versionNameRegex, in: gradlePropertiesContents) else {
            Logger.printFatalError("VERSION_NAME variable was not found in gradle.properties!")
            return ""
        }
        
        let versionNumberRegex = "[1-9]+.[0-9]+.[0-9]+"
        guard let versionNumber = Helpers.matches(for: versionNumberRegex, in: versionName) else {
            Logger.printFatalError("No version number was found in gradle.properties!")
            return ""
        }
        
        return versionNumber
    }
    
}
